------------------------------------------------------------------------
--                    CREACI DE TAULES, 
------------------------------------------------------------------------


-- TAULA ALUMNES

CREATE TABLE ALUMNE (
            NEXPALUMNE NUMERIC(9) CONSTRAINT PK_ALUMNE PRIMARY KEY,
            nif VARCHAR2(9 CHAR) CONSTRAINT NN_ALUMNE_nif NOT NULL,
								 CONSTRAINT UN_ALUMNE UNIQUE (nif),
            nomCognoms VARCHAR2(160 CHAR) CONSTRAINT NN_ALUMNE_nomCognoms NOT NULL,
            adreca VARCHAR2(160 CHAR) CONSTRAINT NN_ALUMNE_adreca NOT NULL,
            telefon VARCHAR2(9 CHAR),
            email VARCHAR2(50 CHAR));

CREATE SEQUENCE seq_ALUMNE INCREMENT BY 1 START WITH 1000;

CREATE OR REPLACE
TRIGGER inserir_NEXPALUMNE_ALUMNE
    BEFORE INSERT ON ALUMNE
    FOR EACH ROW
BEGIN
    SELECT seq_ALUMNE.NEXTVAL INTO :NEW.NEXPALUMNE
    FROM DUAL;
END inserir_NEXPALUMNE_ALUMNE;
/

-- TAULA ASSIGNATURA
CREATE TABLE ASSIGNATURA (
            IDASSIG NUMERIC(5) CONSTRAINT PK_ASSIGNATURA PRIMARY KEY,
            nomAssignatura VARCHAR2(20 CHAR) CONSTRAINT NN_ASSIGNATURA_nomAssignatura NOT NULL,
                                           CONSTRAINT UN_ASSIGNATURAO UNIQUE (nomAssignatura),
            descripcio VARCHAR2(160 CHAR) CONSTRAINT NN_ASSIGNATURA_descripcio NOT NULL);


CREATE SEQUENCE seq_ASSIGNATURA INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE
TRIGGER inserir_IDASSIG_ASSIGNATURA
    BEFORE INSERT ON ASSIGNATURA
    FOR EACH ROW
BEGIN
    SELECT seq_ASSIGNATURA.NEXTVAL INTO :NEW.IDASSIG
    FROM DUAL;
END inserir_IDASSIG_ASSIGNATURA;
/ 

-- TAULA CURS

CREATE TABLE CURS (
            IDCURS NUMERIC(9) CONSTRAINT PK_CURS PRIMARY KEY,
            nomCurs VARCHAR2(15 CHAR) CONSTRAINT NN_CURS_nomCurs NOT NULL,
            anyCurs NUMBER(4) CONSTRAINT NN_CURS_anyCurs NOT NULL,
            descripcio VARCHAR2(160 CHAR) CONSTRAINT NN_CURS_descripcio NOT NULL,
            dataInici DATE CONSTRAINT NN_CURS_dataInici NOT NULL,
            dataFi DATE CONSTRAINT NN_CURS_dataFi NOT NULL,
            CONSTRAINT UN_CURS UNIQUE (nomCurs,anyCurs));
            
CREATE SEQUENCE seq_CURS INCREMENT BY 1 START WITH 1000;

CREATE OR REPLACE
TRIGGER inserir_IDCURS_CURS
    BEFORE INSERT ON CURS
    FOR EACH ROW
BEGIN
    SELECT seq_CURS.NEXTVAL INTO :NEW.IDCURS
    FROM DUAL;
END inserir_IDCURS_CURS;
/

-- TAULA CURS_ALUMNE

CREATE TABLE CURS_ALUMNE (
             NEXPALUMNE CONSTRAINT FK1_CURS_ALUMNE REFERENCES ALUMNE(NEXPALUMNE),
             IDCURS CONSTRAINT FK2_CURS_ALUMNE REFERENCES CURS(IDCURS),
             CONSTRAINT PK_CURS_ALUMNE PRIMARY KEY (NEXPALUMNE,IDCURS));
/
             
-- TAULA PROFESSOR
			 
CREATE TABLE PROFESSOR (
            NEXPPROFESSOR NUMERIC(9) CONSTRAINT PK_PROFESSOR PRIMARY KEY,
            nif VARCHAR2(9 CHAR) CONSTRAINT NN_PROFESSOR_nif NOT NULL,
								CONSTRAINT UN_PROFESSOR UNIQUE (nif),
            nomCognoms VARCHAR2(160 CHAR) CONSTRAINT NN_PROFESSOR_nomCognoms NOT NULL,
            adreca VARCHAR2(160 CHAR) CONSTRAINT NN_PROFESSOR_adreca NOT NULL,
            telefon VARCHAR2(9 CHAR),
            email VARCHAR2(50 CHAR),
            tutorCurs CONSTRAINT FK_PROFESSOR REFERENCES CURS(IDCURS));

CREATE SEQUENCE seq_PROFESSOR INCREMENT BY 1 START WITH 1000;

CREATE OR REPLACE
TRIGGER inserir_NEXPPROFESSOR_PROF
    BEFORE INSERT ON PROFESSOR
    FOR EACH ROW
BEGIN
    SELECT seq_PROFESSOR.NEXTVAL INTO :NEW.NEXPPROFESSOR
    FROM DUAL;
END inserir_NEXPPROFESSOR_PROF;
/

-- TAULA T_AMONESTACIO

CREATE TABLE T_AMONESTACIO (
            IDTIPUSAMO NUMERIC(5) CONSTRAINT PK_T_AMONESTACIO PRIMARY KEY,
            tAmonestacio VARCHAR2(20 CHAR) CONSTRAINT NN_T_AMONESTACIO_tAmonestacio NOT NULL,
                                           CONSTRAINT UN_T_AMONESTACIO UNIQUE (tAmonestacio),
            descripcio VARCHAR2(160 CHAR) CONSTRAINT NN_T_AMONESTACIO_descripcio NOT NULL);

CREATE SEQUENCE seq_T_AMONESTACIO INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE
TRIGGER inserir_IDTIPUSAMO_T_AMON
    BEFORE INSERT ON T_AMONESTACIO
    FOR EACH ROW
BEGIN
    SELECT seq_T_AMONESTACIO.NEXTVAL INTO :NEW.IDTIPUSAMO
    FROM DUAL;
END inserir_IDTIPUSAMO_T_AMON;
/

-- TAULA T_SANCIO

CREATE TABLE T_SANCIO (
            IDTIPUSSANCIO NUMERIC(5) CONSTRAINT PK_T_SANCIO PRIMARY KEY,
            tSancio VARCHAR2(160 CHAR) CONSTRAINT NN_T_SANCIO_tSancio NOT NULL,
                                           CONSTRAINT UN_T_SANCIO UNIQUE (tSancio),
            quanActiva VARCHAR2(160 CHAR) CONSTRAINT NN_T_SANCIO_quanActiva NOT NULL,
            tAmonestacio CONSTRAINT FK_T_SANCIO REFERENCES T_AMONESTACIO(IDTIPUSAMO),
            operador CHAR(1),
            nAmonestacions NUMERIC (3) DEFAULT 1);


CREATE SEQUENCE seq_T_SANCIO INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE
TRIGGER inserir_IDTIPUSSANCIO_T_SANCIO
    BEFORE INSERT ON T_SANCIO
    FOR EACH ROW
BEGIN
    SELECT seq_T_SANCIO.NEXTVAL INTO :NEW.IDTIPUSSANCIO
    FROM DUAL;
END inserir_IDTIPUSSANCIO_T_SANCIO;
/

-- TAULA SANCIO

CREATE TABLE SANCIO (
            IDSANCIO NUMERIC(5) CONSTRAINT PK_SANCIO PRIMARY KEY,
            nExpAlumne CONSTRAINT NN_SANCIO_nExpAlumne NOT NULL
                        CONSTRAINT FK1_SANCIO REFERENCES ALUMNE(NEXPALUMNE),
            tipus CONSTRAINT NN_SANCIO_tipus NOT NULL
                  CONSTRAINT FK2_SANCIO REFERENCES T_SANCIO(IDTIPUSSANCIO),
            data DATE CONSTRAINT NN_SANCIO_data NOT NULL,
            hora VARCHAR(8 CHAR) CONSTRAINT NN_SANCIO_hora NOT NULL,
            curs CONSTRAINT NN_SANCIO_curs NOT NULL
                 CONSTRAINT FK3_SANCIO REFERENCES CURS(IDCURS),
            descripcio VARCHAR(160 CHAR),
            CONSTRAINT UN_SANCIO UNIQUE (nExpAlumne,tipus,data,hora));
            
CREATE INDEX IDX_SANCIO ON SANCIO (nExpAlumne);           
            
CREATE SEQUENCE seq_SANCIO INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE
TRIGGER inserir_IDSANCIO_SANCIO
    BEFORE INSERT ON SANCIO
    FOR EACH ROW
BEGIN
    SELECT seq_SANCIO.NEXTVAL INTO :NEW.IDSANCIO
    FROM DUAL;
END inserir_IDSANCIO_SANCIO;
/

-- TAULA AMONESTACIO

CREATE TABLE AMONESTACIO(
            IDAMON NUMERIC(5) CONSTRAINT PK_AMONESTACIO PRIMARY KEY,
            nExpAlumne CONSTRAINT NN_AMONESTACIO_nExpAlumne NOT NULL
                        CONSTRAINT FK1_AMONESTACIO REFERENCES ALUMNE(NEXPALUMNE),
            tAmon CONSTRAINT NN_AMONESTACIO_tAmon NOT NULL
                  CONSTRAINT FK2_AMONESTACIO REFERENCES T_AMONESTACIO(IDTIPUSAMO),
            assignatura CONSTRAINT NN_AMONESTACIO_assignatura NOT NULL
                  CONSTRAINT FK3_AMONESTACIO REFERENCES ASSIGNATURA(IDASSIG),
            profResponsable CONSTRAINT NN_AMONESTACIO_profResponsable NOT NULL
                  CONSTRAINT FK4_AMONESTACIO REFERENCES PROFESSOR(NEXPPROFESSOR),  
            curs CONSTRAINT NN_AMONESTACIO_curs NOT NULL
                 CONSTRAINT FK5_AMONESTACIO REFERENCES CURS(IDCURS),  
            sancio CONSTRAINT FK6_AMONESTACIO REFERENCES SANCIO(IDSANCIO),         
            data DATE CONSTRAINT NN_AMONESTACIO_data NOT NULL,
            hora VARCHAR(8 CHAR) CONSTRAINT NN_AMONESTACIO_hora NOT NULL,
            gravetat CHAR(1 BYTE) DEFAULT 1 CONSTRAINT NN_AMONESTACIO_gravetat NOT NULL,
            comPares CHAR(1 BYTE) DEFAULT 0 CONSTRAINT NN_AMONESTACIO_comPares NOT NULL,
            descripcio VARCHAR(160 CHAR),
            CONSTRAINT UN_AMONESTACIO UNIQUE (nExpAlumne,tAmon,data,hora),
			CONSTRAINT CHK1_AMONESTACIO CHECK (gravetat BETWEEN 0 AND 9),
			CONSTRAINT CHK2_AMONESTACIO CHECK (comPares=0 OR comPares=1));

CREATE INDEX IDX_AMONESTACIO ON AMONESTACIO (nExpAlumne);			
 
CREATE SEQUENCE seq_AMONESTACIO INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE
TRIGGER inserir_IDAMON_AMONESTACIO
    BEFORE INSERT ON AMONESTACIO
    FOR EACH ROW
BEGIN
    SELECT seq_AMONESTACIO.NEXTVAL INTO :NEW.IDAMON
    FROM DUAL;
END inserir_IDAMON_AMONESTACIO;
/
        
-- TAULA IMPARTEIX
			
CREATE TABLE IMPARTEIX(
            assignatura CONSTRAINT NN_IMPARTEIX_assignatura NOT NULL
                  CONSTRAINT FK1_IMPARTEIX REFERENCES ASSIGNATURA(IDASSIG),
            professor CONSTRAINT NN_IMPARTEIX_professor NOT NULL
                  CONSTRAINT FK2_IMPARTEIX REFERENCES PROFESSOR(NEXPPROFESSOR),  
            curs CONSTRAINT NN_IMPARTEIX_curs NOT NULL
                 CONSTRAINT FK3_IMPARTEIX REFERENCES CURS(IDCURS),
                 CONSTRAINT PK_IMPARTEIX PRIMARY KEY (assignatura,curs));

-- TAULA DIESLECTIUS
 
CREATE TABLE DIESLECTIUS(
            DATA DATE CONSTRAINT PK_DIESLECTIUS PRIMARY KEY,
            diaSetmana CHAR(1) CONSTRAINT NN_DIESLECTIUS_diaSetmana NOT NULL);                 

-- TAULA CALENDARI			
			
CREATE TABLE CALENDARI(
            assignatura CONSTRAINT NN_CALENDARI_assignatura NOT NULL
                 CONSTRAINT FK1_CALENDARI REFERENCES ASSIGNATURA(IDASSIG),
            curs CONSTRAINT NN_CALENDARI_curs NOT NULL
                 CONSTRAINT FK2_CALENDARI REFERENCES CURS(IDCURS),
            dia CONSTRAINT NN_CALENDARI_dia NOT NULL
                 CONSTRAINT FK3_CALENDARI REFERENCES DIESLECTIUS(DATA) ON DELETE CASCADE,
            hora VARCHAR(8 CHAR) CONSTRAINT NN_CALENDARI_hora NOT NULL,
			diaSetmana CHAR(1) CONSTRAINT NN_CALENDARI_diaSetmana NOT NULL,
            CONSTRAINT PK_CALENDARI PRIMARY KEY (curs,dia,hora));
/
			
-- TAULA LOG
         
CREATE TABLE LOG_ERROR(
            IDLOG NUMERIC(9) CONSTRAINT PK_LOG PRIMARY KEY,
            PROCEDIMENT VARCHAR (30 CHAR)CONSTRAINT NN_LOG_PROCEDIMENT NOT NULL,
            DATA DATE CONSTRAINT NN_LOG_DATA NOT NULL,
            paramEntrada  VARCHAR (255 CHAR),
            paramSortida   VARCHAR (255 CHAR),
            rsp VARCHAR (150 CHAR));


CREATE SEQUENCE seq_LOG INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE
TRIGGER inserir_IDLOG_LOG
    BEFORE INSERT ON LOG_ERROR
    FOR EACH ROW
BEGIN
    SELECT seq_LOG.NEXTVAL INTO :NEW.IDLOG
    FROM DUAL;
END inserir_IDLOG_LOG;
/
            
-- TAULA ESTADISTIC
			
CREATE TABLE ESTADISTIC(
            idEstad NUMBER (9) CONSTRAINT PK_ESTADISTIC PRIMARY KEY,
            nomEntitat1 VARCHAR2 (15 CHAR),
            nomEntitat2 VARCHAR2 (15 CHAR),
            ID1 NUMERIC (9),
            ID2 NUMERIC (9),
            nAmonestacions  NUMERIC (3) DEFAULT 0,
            nSancions   NUMERIC (3) DEFAULT 0,
            mitjanaAmo  NUMERIC (3) DEFAULT 0,
            alumne CONSTRAINT FK1_ESTADISTIC REFERENCES ALUMNE(NEXPALUMNE),
            professor CONSTRAINT FK2_ESTADISTIC REFERENCES PROFESSOR(NEXPPROFESSOR),  
            mitjanaSancions  NUMERIC (3) DEFAULT 0,
            nAlumnes NUMERIC (5) DEFAULT 0,
			nCursos NUMBER(5) DEFAULT 0,
			nomProfessor  VARCHAR2(100 CHAR),
			nomAlumne VARCHAR2(100 CHAR),
            CONSTRAINT UN_ESTADISTIC UNIQUE (nomEntitat1,nomEntitat2,ID1,ID2),
            CONSTRAINT CHK_ESTADISTIC1 CHECK ((nomEntitat1 IS NOT NULL AND ID1 IS NOT NULL) OR (nomEntitat1 IS NULL AND ID1 IS NULL)),
            CONSTRAINT CHK_ESTADISTIC2 CHECK ((nomEntitat2 IS NOT NULL AND ID2 IS NOT NULL) OR (nomEntitat2 IS NULL AND ID2 IS NULL)),
            CONSTRAINT CHK_ESTADISTIC3 CHECK ((nomEntitat1 IS NOT NULL AND ID1 IS NOT NULL) OR (nomEntitat2 IS NOT NULL AND ID2 IS NOT NULL)));
            
            
 CREATE SEQUENCE seq_ESTADISTIC INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE
TRIGGER inserir_IDESTAD_ESTADISTIC
    BEFORE INSERT ON ESTADISTIC
    FOR EACH ROW
BEGIN
    SELECT seq_ESTADISTIC.NEXTVAL INTO :NEW.IDESTAD
    FROM DUAL;
END inserir_IDESTAD_ESTADISTIC;
/			
			

