
-- Alta d'un alumne --
CREATE OR REPLACE PROCEDURE Alta_Alumne (v_nif IN ALUMNE.nif%TYPE,
            v_nomCognoms IN ALUMNE.nomCognoms%TYPE, 
            v_adreca IN ALUMNE.adreca%TYPE,
             v_telefon IN ALUMNE.telefon%TYPE,
             v_email IN ALUMNE.email%TYPE, 
             rsp OUT NOCOPY VARCHAR2) IS

    textLog VARCHAR2(255 CHAR);
    nExpAlumneNou ALUMNE.nExpAlumne%TYPE;
    
    BEGIN
        BEGIN
        
            INSERT INTO ALUMNE (NIF,NOMCOGNOMS, ADRECA, TELEFON, EMAIL) 
            VALUES (v_nif,v_nomCognoms,v_adreca,v_telefon,v_email) RETURNING nExpAlumne INTO nExpAlumneNou;   
      
            --Actualitzaci mdul estadstic 
           Gestio_Estadistic ('Alumne',nExpAlumneNou,NULL,NULL,'Alta_Alumne',NULL, RSP);
            
            rsp:='OK' || nExpAlumneNou; 
            COMMIT;

        EXCEPTION
            WHEN DUP_VAL_ON_INDEX THEN
                rsp:= ('Error:DUP_VAL_ON_INDEX : L''alumne ja existeix');
                ROLLBACK; 
            WHEN OTHERS THEN 
                rsp:='Error:'||SQLERRM;
                ROLLBACK;
        END;    
            -- anotar a la taula de log
        textLog:=('nExpAlumne=' || nExpAlumneNou || ', nif='|| v_nif || ', nomCognoms=' || v_nomCognoms || ', adreca=' ||v_adreca ||
             ', telefon=' || v_telefon || ', email=' ||  v_email); 
        insert_log('Alta_Alumne', textLog, '', rsp);
    END;
/
   
-- Alta d'una Assignatura
CREATE OR REPLACE PROCEDURE Alta_Assignatura (v_nomAssignatura IN ASSIGNATURA.nomAssignatura%TYPE,
            v_descripcio IN ASSIGNATURA.descripcio%TYPE, 
             rsp OUT NOCOPY VARCHAR2) IS
    textLog VARCHAR2(255 CHAR);
    BEGIN        
        BEGIN
            INSERT INTO ASSIGNATURA (NOMASSIGNATURA,DESCRIPCIO) 
            VALUES (v_nomAssignatura,v_descripcio);   
            rsp:='OK';  
            COMMIT;
        EXCEPTION
             WHEN DUP_VAL_ON_INDEX THEN
                rsp:= ('Error:DUP_VAL_ON_INDEX : L''assignatura ja existeix');
                ROLLBACK;
            WHEN OTHERS THEN 
                rsp:='Error:'||SQLERRM;
                ROLLBACK;    
        END;    
                -- anotar a la taula de log
        textLog:=('nomAssignatura='|| v_nomAssignatura || ', descripcio=' || v_descripcio); 
        insert_log('Alta_Assignatura', textLog, '', rsp);
    END;
/    
-- Alta d'un Curs
CREATE OR REPLACE PROCEDURE Alta_Curs (v_nomCurs IN CURS.nomCurs%TYPE,
            v_anyCurs IN CURS.anyCurs%TYPE, 
            v_descripcio IN CURS.descripcio%TYPE,
             v_dataInici IN CURS.dataInici%TYPE,
             v_dataFi IN CURS.dataFi%TYPE, 
             rsp OUT NOCOPY VARCHAR2) IS
             
    textLog VARCHAR2(255 CHAR);
    DATA_INICI_INCORRECTA EXCEPTION;
    DATA_FI_INCORRECTA EXCEPTION;
    ANYCURS_INCORRECTE EXCEPTION;
    idCursNou CURS.idCurs%TYPE;
    
    BEGIN
        BEGIN
			-- comprovaci de dates
            IF TO_DATE (v_dataInici, 'DD/MM/YYYY') < TO_DATE(SYSDATE,'DD/MM/YYYY') THEN RAISE DATA_INICI_INCORRECTA;
            END IF;
            IF TO_DATE (v_dataFi, 'DD/MM/YYYY') < TO_DATE (v_dataInici, 'DD/MM/YYYY') THEN RAISE DATA_FI_INCORRECTA;
            END IF;
            IF TRUNC(TO_DATE (v_anyCurs, 'YYYY'), 'YEAR') < TRUNC(SYSDATE,'YEAR') THEN RAISE ANYCURS_INCORRECTE;
            END IF;
  
            INSERT INTO CURS (NOMCURS,ANYCURS, DESCRIPCIO, DATAINICI, DATAFI)
                VALUES (v_nomCurs,v_anyCurs,v_descripcio,v_dataInici,v_dataFi) 
                RETURNING idCurs INTO idCursNou;   
            
            --Actualitzaci mdul estadstic 
            Gestio_Estadistic ('Curs',idCursNou,NULL,NULL,'Alta_Curs',NULL, RSP);

            rsp:='OK';  
            COMMIT;
            
        EXCEPTION
            WHEN DATA_INICI_INCORRECTA THEN
                rsp:= ('Error:DATA_INICI_INCORRECTA : La data d''inici del curs no s correcta');
                 ROLLBACK; 
            WHEN DATA_FI_INCORRECTA THEN
                rsp:= ('Error:DATA_FI_INCORRECTA : La data de fi del curs no s correcta');
                 ROLLBACK; 
            WHEN ANYCURS_INCORRECTE THEN
                rsp:= ('Error:ANYCURS_INCORRECTE : L''any del curs no s correcte');
                 ROLLBACK;
            WHEN DUP_VAL_ON_INDEX THEN
                rsp:= ('Error:DUP_VAL_ON_INDEX : El curs ja existeix');
                 ROLLBACK; 
         
            WHEN OTHERS THEN 
                rsp:='Error:'||SQLERRM;
                ROLLBACK;
        END;    
            -- anotar a la taula de log
        textLog:=('nomCurs='|| v_nomCurs || ', anyCurs=' || v_anyCurs || ', descripcio=' ||v_descripcio ||
             ', dataInici=' || v_dataInici || ', dataFi=' ||  v_dataFi); 
        insert_log('Alta_Curs', textLog, '', rsp);
    END;
/   
   
-- Alta d'un alumne a un curs
CREATE OR REPLACE PROCEDURE Alta_CursAlumne (v_nExpAlumne IN CURS_ALUMNE.nExpAlumne%TYPE,
                                     v_idCurs IN CURS_ALUMNE.idCurs%TYPE,
                                     rsp OUT NOCOPY VARCHAR2) IS
    textLog VARCHAR2(255 CHAR);
    CURS_FINALITZAT EXCEPTION;
    cursRow curs%ROWTYPE;
    
    BEGIN
    
        BEGIN
                -- El curs ha finalitzat?
            SELECT * INTO cursRow FROM CURS WHERE idCurs = v_idCurs;
            IF TO_DATE(cursRow.dataFi,'DD/MM/YYYY') < TO_DATE(SYSDATE,'DD/MM/YYYY') THEN RAISE CURS_FINALITZAT;
            END IF;
            
            INSERT INTO CURS_ALUMNE (nExpAlumne, idCurs)
            VALUES (v_nExpAlumne, v_idCurs);
            
            --Actualitzaci mdul estadstic 
            Gestio_Estadistic ('Alumne',v_nExpAlumne,'Curs',v_idCurs,'Alta_CursAlumne',NULL, RSP);
            
            rsp:='OK';  
            COMMIT;

        EXCEPTION
            WHEN CURS_FINALITZAT THEN
                rsp:= ('Error:CURS_FINALITZAT : El curs ja ha finalitzat');
                ROLLBACK; 
            WHEN DUP_VAL_ON_INDEX THEN
                rsp:= ('Error:DUP_VAL_ON_INDEX : L''alumne ja est inscrit al curs');
                ROLLBACK; 
			WHEN NO_DATA_FOUND THEN
                rsp:= 'El curs referit no existeix';
                ROLLBACK;
            WHEN OTHERS THEN 
                IF SQLCODE=-2291 THEN
                    rsp:='Alguna de les dades introduides no existeix';
                    ROLLBACK;
                ELSE
                    rsp:='Error:'||SQLERRM;
                    ROLLBACK;
                END IF;
        END;
            -- anotar a la taula de log
        textLog:=('nExpAlumne='|| v_nExpAlumne || ', idCurs=' || v_idCurs); 
        insert_log('Alta_CursAlumne', textLog, '', rsp);
    END;
/    
    
-- Alta de professor
CREATE OR REPLACE PROCEDURE Alta_Professor (v_nif IN PROFESSOR.nif%TYPE,
            v_nomCognoms IN PROFESSOR.nomCognoms%TYPE, 
            v_adreca IN PROFESSOR.adreca%TYPE,
             v_telefon IN PROFESSOR.telefon%TYPE,
             v_email IN PROFESSOR.email%TYPE, 
             v_tutorCurs IN PROFESSOR.tutorCurs%TYPE,
             rsp OUT NOCOPY VARCHAR2) IS
             
    textLog VARCHAR2(255 CHAR);
    nExpProfessorNou PROFESSOR.nExpProfessor%TYPE;
    
    BEGIN    
        BEGIN
            INSERT INTO PROFESSOR (NIF,NOMCOGNOMS, ADRECA, TELEFON, EMAIL, TUTORCURS) 
                VALUES (v_nif,v_nomCognoms,v_adreca,v_telefon,v_email,v_tutorCurs);
            rsp:='OK';  
            COMMIT;

        EXCEPTION
            
            WHEN DUP_VAL_ON_INDEX THEN
                rsp:= 'Error: DUP_VAL_ON_INDEX : El professor ja existeix';
                ROLLBACK; 
         
            WHEN OTHERS THEN 
            
                IF SQLCODE=-2291 THEN
                    rsp:='Error: CURS_NO_EXISTEIX : El curs que es vol assignar no existeix';
                    ROLLBACK;
                ELSE
                    rsp:='Error:'||SQLERRM;
                    ROLLBACK;
                END IF;
        END;    
        
                    -- anotar a la taula de log
        textLog:=('nif='|| v_nif || ', nomCognoms=' || v_nomCognoms || ', adreca=' || v_adreca ||
             ', telefon=' || v_telefon || ', email=' ||  v_email || ', tutorCurs=' || v_tutorCurs); 
        insert_log('Alta_Professor', textLog, '', rsp);
    END;
/
    
-- Alta d'assignatures a cursos i assignaci de professors 
CREATE OR REPLACE PROCEDURE Alta_Imparteix (v_assignatura IN IMPARTEIX.assignatura%TYPE,
                                     v_professor IN IMPARTEIX.professor%TYPE,
                                     v_curs IN IMPARTEIX.curs%TYPE,
                                     rsp OUT NOCOPY VARCHAR2) IS
    textLog VARCHAR2(255 CHAR);
    CURS_FINALITZAT EXCEPTION;
    cursRow curs%ROWTYPE;
    
    BEGIN
        BEGIN
             -- El curs ha finalitzat?
            SELECT * INTO cursRow FROM CURS WHERE idCurs = v_Curs;
            IF TO_DATE(cursRow.dataFi,'DD/MM/YYYY') < TO_DATE(SYSDATE,'DD/MM/YYYY') THEN RAISE CURS_FINALITZAT;
            END IF;
            
            INSERT INTO IMPARTEIX(assignatura, professor, curs)
                VALUES (v_assignatura, v_professor, v_curs);
                
            --Actualitzaci mdul estadstic 
            Gestio_Estadistic ('Professor',v_professor,'Curs',v_curs,'Alta_Imparteix',NULL, RSP);      
            
            rsp:='OK';  
            COMMIT;
        
        EXCEPTION
            WHEN CURS_FINALITZAT THEN
                rsp:= ('Error: CURS_FINALITZAT : El curs ja ha finalitzat');
                ROLLBACK; 
            WHEN DUP_VAL_ON_INDEX THEN
                rsp:= ('Error: DUP_VAL_ON_INDEX : L''assignatura ja s''est impartint al curs');
                ROLLBACK; 
			WHEN NO_DATA_FOUND THEN
                rsp:= 'El curs referit no existeix';
                ROLLBACK;
            WHEN OTHERS THEN 
            
                IF SQLCODE=-2291 THEN
                    rsp:='Alguna de les dades introduides no existeix';
                    ROLLBACK;
                ELSE
                    rsp:='Error:'||SQLERRM;
                    ROLLBACK;
                END IF;
        END;
               -- anotar a la taula de log
        textLog:=('assignatura='|| v_assignatura || ', professor=' || v_professor || ', curs=' || v_curs); 
        insert_log('Alta_Imparteix', textLog, '', rsp);
        
    END;
/    
    
-- Alta de dies lectius
CREATE OR REPLACE PROCEDURE Alta_DiesLectius (v_dataInici IN VARCHAR2, v_dataFi IN VARCHAR2, rsp OUT NOCOPY VARCHAR2) IS

    dataInici DATE;
    dataFi DATE;
    dataLoop DATE;
    textLog VARCHAR2(255 CHAR);

    BEGIN
        dataInici:= TO_DATE(v_dataInici);
        dataFi:= TO_DATE(v_dataFi);
        
        DECLARE
        seguentDia DATE;
        BEGIN
            
            -- Es donen d'alta els dies lectius entre dataInici i dataFi exceptuant dissabtes i diumenges
            dataLoop := dataInici;
            LOOP
            
            seguentDia:= NEXT_DAY(dataLoop, 'domingo');
            IF (seguentDia - dataLoop) <> 1  AND (seguentDia - dataLoop) <> 7 THEN
            
                INSERT INTO DIESLECTIUS (DATA,DIASETMANA) VALUES (dataLoop,TO_CHAR(dataLoop,'d'));  
                
                -- Comprova si hi ha assignatures que s'nmparteixen en aquell dia i les dona d'alta al calendari
                DECLARE
                CURSOR cursAfegir IS SELECT C.curs, C.assignatura, C.hora FROM CALENDARI C INNER JOIN CURS ON idCurs=curs 
                                        WHERE C.diaSetmana=TO_CHAR(dataLoop,'d') AND dataLoop between CURS.dataInici AND CURS.dataFi 
                                        GROUP BY C.curs, C.assignatura, C.hora;
                cursRow cursAfegir%ROWTYPE;
                
                BEGIN
                    OPEN cursAfegir;
                    LOOP
                        FETCH cursAfegir INTO cursRow;
                        EXIT WHEN cursAfegir%NOTFOUND;
                        INSERT INTO CALENDARI (CURS, ASSIGNATURA, DIA, HORA, DIASETMANA) VALUES (cursRow.curs, cursRow.assignatura, dataLoop, cursRow.hora, TO_CHAR(dataLoop,'d'));
                    END LOOP;
                END;
            END IF;

            dataLoop:=dataLoop +1;
            EXIT WHEN dataLoop >= dataFi;
            
            END LOOP;
                rsp:='OK'; 
            COMMIT;
            
        EXCEPTION
        
            WHEN DUP_VAL_ON_INDEX THEN
                rsp:= 'ERROR: DUP_VAL_ON_INDEX : El dia ' || dataLoop || ' ja estava donat d''alta';
                ROLLBACK; 
             
            WHEN OTHERS THEN 
                rsp:='ERROR:'||SQLERRM;
                ROLLBACK;
       
        END;

                 -- anotar a la taula de log
            textLog:=('dataInici='|| dataInici || ', dataFi=' || dataFi); 
            insert_log('Alta_DiesLectius', textLog, '', rsp);

    END;
/    
-- Alta del calendari
CREATE OR REPLACE PROCEDURE Alta_Calendari (v_curs IN CALENDARI.curs%TYPE,
                    v_assignatura IN CALENDARI.assignatura%TYPE,
                    v_dia IN CALENDARI.diaSetmana%TYPE,
                    v_hora IN CALENDARI.hora%TYPE,
                    rsp OUT NOCOPY VARCHAR2) IS 
    
    textLog VARCHAR2(255 CHAR);
    v_dataInici DATE;
    v_dataFi DATE;
    CURS_COMENCAT EXCEPTION;
    DIESLECTIUS_NO_TROBATS EXCEPTION;
    
    BEGIN
        
        BEGIN
            SELECT dataInici, dataFi INTO v_dataInici, v_dataFi FROM CURS WHERE idCurs=v_curs;

            DECLARE
            CURSOR Dies IS SELECT * FROM DIESLECTIUS WHERE diaSetmana = v_dia AND data between v_dataInici AND v_dataFi;
            diesRow Dies%ROWTYPE;
            
            BEGIN
            -- Comprova que el curs no ha comenat o finalitzat
            IF TO_DATE (v_dataInici, 'DD/MM/YYYY') < TO_DATE(SYSDATE, 'DD/MM/YYYY') THEN RAISE CURS_COMENCAT;
            END IF;
            
            -- Dona d'alta una entrada per cada dia que pertanyi al dia de la setmana indicat
            --  entre dataInici i dataFi
            
            OPEN Dies;
            LOOP
            FETCH Dies INTO diesRow;
            EXIT WHEN dies%NOTFOUND;
                INSERT INTO CALENDARI (CURS, ASSIGNATURA, DIA, HORA, DIASETMANA) VALUES (v_curs, v_assignatura, diesRow.data, v_hora, v_dia);
            
            END LOOP;
            CLOSE Dies;
            
            END;
            
            RSP:='OK';
            COMMIT;
            
        EXCEPTION
            WHEN DUP_VAL_ON_INDEX THEN
                    rsp:= 'ERROR: DUP_VAL_ON_INDEX : Aquest l''entrada del calendari ja existeix';
                    ROLLBACK; 
            WHEN DIESLECTIUS_NO_TROBATS THEN
                RSP:='ERROR: DIESLECTIUS_NO_TROBATS :  Els dieslectius no han estat dotats d''alta';
                ROLLBACK;
            WHEN NO_DATA_FOUND THEN
                rsp:= 'ERROR: NO_DATA_FOUND : Els dieslectius no han estat dotats d''alta';
                ROLLBACK; 
            WHEN CURS_COMENCAT THEN
                RSP:='ERROR: CURS_COMENCAT : El curs ja ha comenat o finalitzat';
                ROLLBACK;
            WHEN OTHERS THEN
                RSP:='ERROR: ' || SQLERRM;
                ROLLBACK;
        END;
        
        -- anotar a la taula de log
        textLog:=('Curs=' || v_curs || 'Assignatura='|| v_assignatura || ', diaSetmana=' || v_dia); 
        insert_log('Alta_Calendari', textLog, '', rsp);
    
    END;
/	

-- Alta de tipus d'amonestaci
CREATE OR REPLACE PROCEDURE Alta_TAmonestacio (v_tAmonestacio IN T_AMONESTACIO.tAmonestacio%TYPE,
            v_descripcio IN T_AMONESTACIO.descripcio%TYPE, 
             rsp OUT NOCOPY VARCHAR2) IS
             
    textLog VARCHAR2(255 CHAR);
    BEGIN        
        BEGIN
        
            INSERT INTO T_AMONESTACIO (TAMONESTACIO,DESCRIPCIO) 
            VALUES (v_tAmonestacio,v_descripcio);   

            rsp:='OK';  
            COMMIT;
            
        EXCEPTION
            WHEN DUP_VAL_ON_INDEX THEN
                rsp:= ('Error: DUP_VAL_ON_INDEX : El tipus d''amonestaci ja existeix');
                ROLLBACK;
            WHEN OTHERS THEN 
                rsp:='Error:'||SQLERRM;
                ROLLBACK;    
        END;    
           -- anotar a la taula de log
        textLog:=('tAmonestacio='|| v_tAmonestacio || ', descripcio=' || v_descripcio); 
        insert_log('Alta_TAmonestacio', textLog, '', rsp);
    END;
/    
    
-- Alta de tipus de sanci
CREATE OR REPLACE PROCEDURE Alta_TSancio (v_tSancio IN T_SANCIO.tSancio%TYPE,
            v_quanActiva IN T_SANCIO.quanActiva%TYPE, 
            v_tAmonestacio IN T_SANCIO.tAmonestacio%TYPE,
            v_operador IN T_SANCIO.operador%TYPE,
            v_nAmonestacions IN T_SANCIO.nAmonestacions%TYPE,
            rsp OUT NOCOPY VARCHAR2) IS

    OPERADOR_NO_VALID EXCEPTION;       
    textLog VARCHAR2(255 CHAR);
    BEGIN        
        BEGIN
            -- Comprovaci dels parmetres d'entrada
            IF v_operador NOT IN ('=','<','>') THEN RAISE OPERADOR_NO_VALID;
            END IF;

            INSERT INTO T_SANCIO (TSANCIO,QUANACTIVA,TAMONESTACIO,OPERADOR,NAMONESTACIONS)
            VALUES (v_tSancio,v_quanActiva,v_tAmonestacio,v_operador,v_nAmonestacions);   

            rsp:='OK';  
            COMMIT;
            
        EXCEPTION
            WHEN DUP_VAL_ON_INDEX THEN
                rsp:= 'Error: DUP_VAL_ON_INDEX : El tipus de sanci ja existeix';
                ROLLBACK;
            WHEN OPERADOR_NO_VALID THEN
                rsp:= 'Error: OPERADOR_NO_VALID : L''operador ha de ser <, > o =';
                ROLLBACK;
            WHEN OTHERS THEN 
                IF SQLCODE=-2291 THEN
                    rsp:='Error: AMONESTACIO_NO_EXISTEIX : El tipus d''amonestaci que es vol assignar no existeix';
                    ROLLBACK;
                ELSE
                    rsp:='Error:'||SQLERRM;
                    ROLLBACK;
                END IF;    
        END;    
                -- anotar a la taula de log
        textLog:=('tSancio='|| v_tSancio || ', quanActiva=' || v_quanActiva || ', tAmonestacio='|| v_tAmonestacio ||
             ', operador=' || v_operador || ', nAmonestacions=' || v_nAmonestacions); 
        insert_log('Alta_TSancio', textLog, '', rsp);
    END;    
    
/    
-- Alta Amonestacio
CREATE OR REPLACE PROCEDURE Alta_Amonestacio (v_nExpAlumne IN AMONESTACIO.nExpAlumne%TYPE,
            v_tAmon IN AMONESTACIO.tAmon%TYPE,
            v_assignatura IN AMONESTACIO.assignatura%TYPE,
            v_profResponsable IN  AMONESTACIO.profResponsable%TYPE,
            v_curs IN AMONESTACIO.curs%TYPE,
            v_data IN AMONESTACIO.data%TYPE,
            v_hora IN AMONESTACIO.hora%TYPE,
            v_gravetat IN AMONESTACIO.gravetat%TYPE,
            v_comPares IN AMONESTACIO.comPares%TYPE,
            v_descripcio IN AMONESTACIO.descripcio%TYPE,
            rsp OUT NOCOPY VARCHAR2) IS
            
    DATAHORA_INCORRECTA EXCEPTION;        
    dataHora date;
    textLog VARCHAR2(255 CHAR);
    cursRow curs%ROWTYPE;
    RSP2 VARCHAR2(255 CHAR);
    idAmonNova AMONESTACIO.idAmon%TYPE;
    
    BEGIN
        BEGIN
            dataHora:= TO_DATE(v_data || ':' || v_hora, 'dd/mm/yyyy:hh:mi:ss');
            -- dataHora no s una data futura
            IF dataHora > TO_DATE(SYSDATE,'dd/mm/yyyy:hh:mi:ss') THEN RAISE DATAHORA_INCORRECTA;
            END IF;
            
            -- dataHora est dins de les dates del curs
            SELECT * INTO cursRow FROM CURS WHERE idCurs = v_curs;
            IF (TO_DATE(cursRow.dataInici,'dd/mm/yyyy') > dataHora) OR (TO_DATE(cursRow.dataFi,'dd/mm/yyyy') < dataHora) THEN RAISE DATAHORA_INCORRECTA;
            END IF;
        
            INSERT INTO AMONESTACIO (nExpAlumne,tAmon, assignatura, profResponsable, curs, data, hora, gravetat, comPares, descripcio) 
            VALUES (v_nExpAlumne,v_tAmon,v_assignatura, v_profResponsable, v_curs, v_data, v_hora, v_gravetat, v_comPares, v_descripcio)
            RETURNING idAmon INTO idAmonNova;   
            
            rsp:='OK';  
            COMMIT;
            
            --Actualitzaci mdul estadstic 
            Gestio_Estadistic (NULL,NULL,NULL,NULL,'Alta_Amonestacio',idAmonNova, RSP2);

             -- Un cop tot ha anat b, es comprova si cal crear una sanci             
            CAL_SANCIO(v_nExpAlumne,v_tAmon,v_data,v_hora,v_curs,RSP2);
           
        EXCEPTION

            WHEN DATAHORA_INCORRECTA THEN
                rsp:= 'ERROR: DATAHORA_INCORRECTA : La data i/o l''hora sn incorrectes';
                ROLLBACK; 
            WHEN NO_DATA_FOUND THEN
                rsp:= 'ERROR: NO_DATA_FOUND : El curs referit no existeix';
                ROLLBACK; 
            WHEN DUP_VAL_ON_INDEX THEN
                rsp:= 'ERROR: DUP_VAL_ON_INDEX : L''amonestaci ja existeix';
                ROLLBACK; 
            WHEN OTHERS THEN 
                IF SQLCODE=-2291 THEN
                    rsp:='Alguna de les dades introduides no existeix';
                    ROLLBACK;
                ELSE
                    rsp:='ERROR:'||SQLERRM;
                    ROLLBACK;
                END IF;
        END;    
        
               -- anotar a la taula de log
        textLog:=('nExpAlumne='|| v_nExpAlumne || ', tAmon=' || v_tAmon || ', assignatura=' || v_assignatura || ', profResponsable=' || v_profResponsable || ', curs=' ||  v_curs ||', data=' || v_data ||
             ', hora=' || v_hora || ', gravetat=' || v_gravetat || ', comPares=' || v_comPares || ', descripcio=' || v_descripcio); 
        insert_log('Alta_Amonestacio', textLog, '', rsp);
    END;
/
-- Alta Sancions
CREATE OR REPLACE PROCEDURE Alta_Sancio (v_nExpAlumne IN SANCIO.nExpAlumne%TYPE,
            v_tipus IN SANCIO.tipus%TYPE,
            v_data IN SANCIO.data%TYPE,
            v_hora IN SANCIO.hora%TYPE,
            v_curs IN SANCIO.curs%TYPE,
            v_descripcio IN SANCIO.descripcio%TYPE,
            rsp OUT NOCOPY VARCHAR2) IS
            
    DATAHORA_INCORRECTA EXCEPTION;        
    dataHora date;
    textLog VARCHAR2(255 CHAR);
    cursRow curs%ROWTYPE;
    idSancioNova SANCIO.idSancio%TYPE;
	RSP2 VARCHAR2(255 CHAR);
    
    BEGIN
        BEGIN
            dataHora:= TO_DATE(v_data || ':' || v_hora, 'dd/mm/yyyy:hh:mi');
            -- dataHora no s una data futura
            IF dataHora > SYSDATE THEN RAISE DATAHORA_INCORRECTA;
            END IF;
            
            -- dataHora est dins de les dates del curs
            SELECT * INTO cursRow FROM CURS WHERE idCurs = v_curs;
            IF (TO_DATE(cursRow.dataInici,'dd/mm/yyyy') > dataHora) OR (TO_DATE(cursRow.dataFi,'dd/mm/yyyy') < dataHora) THEN RAISE DATAHORA_INCORRECTA;
            END IF;
        
            -- Alta de la nova Sanci
            INSERT INTO SANCIO (nExpAlumne,tipus, data, hora, curs, descripcio) 
                VALUES (v_nExpAlumne,v_tipus, v_data, v_hora, v_curs, v_descripcio)
                RETURNING idSancio INTO idSancioNova;   
            
            -- Actualitza les amnonestacions del mateix tipus i no sancionades amb la nova sanci   
            UPDATE AMONESTACIO SET sancio = idSancioNova WHERE SANCIO IS NULL AND tAmon = v_tipus;
			
            rsp:='OK';  
            COMMIT;
            
            --Actualitzaci mdul estadstic 
            Gestio_Estadistic (NULL,NULL,NULL,NULL,'Alta_Sancio',idSancioNova, RSP2);

        EXCEPTION
            WHEN DATAHORA_INCORRECTA THEN
                rsp:= 'ERROR: DATAHORA_INCORRECTA : La data i/o l''hora sn incorrectes';
                ROLLBACK; 
            WHEN NO_DATA_FOUND THEN
                rsp:= 'ERROR: NO_DATA_FOUND : El curs referit no existeix';
                ROLLBACK; 
            WHEN DUP_VAL_ON_INDEX THEN
                rsp:= 'ERROR: DUP_VAL_ON_INDEX : La sanci ja existeix';
                ROLLBACK; 
            WHEN OTHERS THEN 
                IF SQLCODE=-2291 THEN
                    rsp:='ERROR: Alguna de les dades introduides no existeix';
                    ROLLBACK;
                ELSE
                    rsp:='ERROR:'||SQLERRM;
                    ROLLBACK;
                END IF;
        END;    
               -- anotar a la taula de log
        textLog:=('nExpAlumne='|| v_nExpAlumne || ', tipus=' || v_tipus || ', data=' || v_data ||
             ', hora=' || v_hora || ', curs=' ||  v_curs || ', descripcio=' || v_descripcio); 
        insert_log('Alta_Sancio', textLog, '', rsp);
    END;
 /
 
-- Baixa Alumne
CREATE OR REPLACE PROCEDURE Baixa_Alumne (v_nExpAlumne IN ALUMNE.nExpAlumne%TYPE,
            rsp OUT NOCOPY VARCHAR2) IS

    textLog VARCHAR2(255 CHAR);
    ALUMNE_NO_TROBAT EXCEPTION;
    
    BEGIN
                
        BEGIN
        
            DELETE FROM ALUMNE WHERE nExpAlumne = v_nExpAlumne;
            IF sql%rowcount = 0 THEN RAISE ALUMNE_NO_TROBAT;
            END IF;

            --Actualitzaci mdul estadstic 
            Gestio_Estadistic ('Alumne',v_nExpAlumne ,NULL,NULL,'Baixa_Alumne',NULL, RSP);

            rsp:='OK';  
            COMMIT;

        EXCEPTION
        
            WHEN ALUMNE_NO_TROBAT THEN
                rsp:= ('ERROR: ALUMNE_NO_TROBAT : L''alumne NO existeix');
                ROLLBACK; 
         
            WHEN OTHERS THEN 
                rsp:='Error:'||SQLERRM;
                ROLLBACK;
                
        END;    
        
              -- anotar a la taula de log
        textLog:=('nExpAlumne' || V_nExpAlumne); 
        insert_log('Baixa_Alumne', textLog, '', rsp);
    END;
/
    
-- Baixa Assignatura
CREATE OR REPLACE PROCEDURE Baixa_Assignatura (v_IDASSIG IN ASSIGNATURA.IDASSIG%TYPE, 
             rsp OUT NOCOPY VARCHAR2) IS
             
    textLog VARCHAR2(255 CHAR);
    ASSIGNATURA_NO_TROBADA EXCEPTION;
    BEGIN        
        BEGIN
        
            DELETE FROM ASSIGNATURA WHERE IDASSIG=v_IDASSIG;   
            IF sql%rowcount = 0 THEN RAISE ASSIGNATURA_NO_TROBADA;
            END IF;

            rsp:='OK';  
            COMMIT;
        EXCEPTION
        
            WHEN ASSIGNATURA_NO_TROBADA THEN
                rsp:= ('ERROR: ASSIGNATURA_NO_TROBADA : L''assignatura NO existeix');
                ROLLBACK; 
                
            WHEN OTHERS THEN 
                rsp:='ERROR:'||SQLERRM;
                ROLLBACK;    
        END;    
        
                -- anotar a la taula de log
        textLog:=('idAssig' || v_IDASSIG); 
        insert_log('Baixa_Assignatura', textLog, '', rsp);
    END;
/    
    
-- Baixa d'un alumne d'un curs
CREATE OR REPLACE PROCEDURE Baixa_CursAlumne (v_nExpAlumne IN CURS_ALUMNE.nExpAlumne%TYPE,
                                     v_idCurs IN CURS_ALUMNE.idCurs%TYPE,
                                     rsp OUT NOCOPY VARCHAR2) IS
    textLog VARCHAR2(255 CHAR);
    CURS_COMENCAT EXCEPTION;
    ALUMNE_CURS_NO_TROBAT EXCEPTION;
    cursRow curs%ROWTYPE;
    
    BEGIN
    
        BEGIN
            -- El curs ha comenat?
            SELECT * INTO cursRow FROM CURS WHERE idCurs = v_idCurs;
            IF cursRow.dataInici < SYSDATE THEN RAISE CURS_COMENCAT;
            END IF;
            
            DELETE FROM CURS_ALUMNE WHERE nExpAlumne=v_nExpAlumne AND idCurs=v_idCurs;
            IF sql%rowcount = 0 THEN RAISE ALUMNE_CURS_NO_TROBAT;
            END IF;
            
            --Actualitzaci mdul estadstic 
            Gestio_Estadistic ('Alumne',v_nExpAlumne,'Curs',v_idCurs,'Baixa_CursAlumne',NULL, RSP);
            
            rsp:='OK';  
            COMMIT;
        
        EXCEPTION
            WHEN CURS_COMENCAT THEN
                rsp:= ('ERROR: CURS_COMENCAT : El curs ja ha comenat');
                ROLLBACK; 
        
            WHEN ALUMNE_CURS_NO_TROBAT THEN
                rsp:= ('ERROR: ALUMNE_CURS_NO_TROBAT : L''alumne no est inscrit al curs');
                ROLLBACK; 
         
            WHEN OTHERS THEN 
                rsp:='Error:'||SQLERRM;
                ROLLBACK;
        END;
        
                
               -- anotar a la taula de log
        textLog:=('nExpAlumne='|| v_nExpAlumne || ', idCurs=' || v_idCurs); 
        insert_log('Baixa_CursAlumne', textLog, '', rsp);
        
    END;
/
    
-- Baixa d'un Curs
CREATE OR REPLACE PROCEDURE Baixa_Curs (v_idCurs IN CURS.idCurs%TYPE,
            rsp OUT NOCOPY VARCHAR2) IS
             
    textLog VARCHAR2(255 CHAR);
    CURS_COMENCAT EXCEPTION;
    v_dataInici CURS.dataInici%TYPE;
    
    BEGIN
                
        BEGIN
        
            SELECT dataInici INTO v_dataInici FROM CURS WHERE idCurs=v_idCurs; 
            IF TO_DATE (v_dataInici, 'DD/MM/YYYY') < TO_DATE (SYSDATE, 'DD/MM/YYYY') THEN RAISE CURS_COMENCAT;
            END IF;
  
            DELETE FROM CURS WHERE idCurs=v_idCurs;  
            
            --Actualitzaci mdul estadstic 
            Gestio_Estadistic ('Curs',v_idCurs,NULL,NULL,'Baixa_Curs',NULL, RSP);
            
            rsp:='OK';  
            COMMIT;

        EXCEPTION
            WHEN CURS_COMENCAT THEN
                rsp:= ('ERROR: CURS_COMENCAT : El curs ja ha comenat');
                ROLLBACK; 
			WHEN NO_DATA_FOUND THEN
                rsp:= 'ERROR: NO_DATA_FOUND : El curs referit no existeix';
                ROLLBACK;
            WHEN OTHERS THEN 
                rsp:='Error:'||SQLERRM;
                ROLLBACK;
        END;    
            -- anotar a la taula de log
        textLog:=('idCurs' || v_idCurs); 
        insert_log('Baixa_Curs', textLog, '', rsp);
    END;
/
-- Baixa d'una Assignatura a un Curs
CREATE OR REPLACE PROCEDURE Baixa_Imparteix (v_assignatura IN IMPARTEIX.assignatura%TYPE,
                                     v_curs IN IMPARTEIX.curs%TYPE,
                                     rsp OUT NOCOPY VARCHAR2) IS
    textLog VARCHAR2(255 CHAR);
    CURS_INICIAT EXCEPTION;
    CURS_ASSIGNATURA_NO_TROBAT EXCEPTION;
    cursRow curs%ROWTYPE;
    v_professor IMPARTEIX.professor%TYPE;
   
    BEGIN
    
        BEGIN
                -- El curs ha finalitzat?
            SELECT * INTO cursRow FROM CURS WHERE idCurs = v_Curs;
            IF cursRow.dataInici < SYSDATE THEN RAISE CURS_INICIAT;
            END IF;
            
            SELECT professor INTO v_professor FROM IMPARTEIX WHERE curs=v_curs AND assignatura= v_assignatura;
            
            DELETE FROM IMPARTEIX WHERE curs=v_curs AND assignatura= v_assignatura;
            IF sql%rowcount = 0 THEN RAISE CURS_ASSIGNATURA_NO_TROBAT;
            END IF;
			
            --Actualitzaci mdul estadstic 
            Gestio_Estadistic ('Professor',v_professor,'Curs',v_curs,'Baixa_Imparteix',NULL, RSP);
            
            rsp:='OK';  
            COMMIT;
        
        EXCEPTION
            WHEN CURS_INICIAT THEN
                rsp:= ('ERROR: CURS_INICIAT : El curs ja s''ha iniciat');
                ROLLBACK; 
            WHEN CURS_ASSIGNATURA_NO_TROBAT THEN
                rsp:= ('ERROR: CURS_ASSIGNATURA_NO_TROBAT : L''assignatura no s''est impartint al curs');
                ROLLBACK; 
            WHEN OTHERS THEN 
                IF SQLCODE=-2291 THEN
                    rsp:='Alguna de les dades introduides no existeix';
                    ROLLBACK;
                ELSE
                    rsp:='Error:'||SQLERRM;
                    ROLLBACK;
                END IF;
        END;
           -- anotar a la taula de log
        textLog:=('assignatura='|| v_assignatura || ', curs=' || v_curs); 
        insert_log('Baixa_Imparteix', textLog, '', rsp);
        
    END;
/    
-- Baixa de dies lectius
CREATE OR REPLACE PROCEDURE Baixa_DiesLectius (v_dataInici IN VARCHAR2,
            v_dataFi IN VARCHAR2,
            rsp OUT NOCOPY VARCHAR2) IS

    dataInici DATE;
    dataFi DATE;
    textLog VARCHAR2(255 CHAR);

    BEGIN
        dataInici:= TO_DATE(v_dataInici,'DD/MM/YYYY');
        dataFi:= NVL(TO_DATE(v_dataFi,'DD/MM/YYYY'),dataInici);
       
        BEGIN
            -- Es donen de baixa els dies lectius entre dataInici i dataFi 
            DELETE FROM DIESLECTIUS WHERE DATA BETWEEN dataInici and dataFi;   
            rsp:='OK'; 
            COMMIT;
            
        EXCEPTION
            WHEN OTHERS THEN 
                rsp:='Error:'||SQLERRM;
                ROLLBACK;
        END;
             -- anotar a la taula de log
        textLog:=('dataInici='|| dataInici || ', dataFi=' || dataFi); 
        insert_log('Baixa_DiesLectius', textLog, '', rsp);
    END;
/	
-- Baixa Calendari	
CREATE OR REPLACE PROCEDURE Baixa_Calendari (v_curs IN CALENDARI.curs%TYPE,
                    v_assignatura IN CALENDARI.assignatura%TYPE,
                    v_dia IN CALENDARI.diaSetmana%TYPE,
                    v_hora IN CALENDARI.hora%TYPE,
                    rsp OUT NOCOPY VARCHAR2) IS 

    textLog VARCHAR2(255 CHAR);
    v_dataInici DATE;
    v_dataFi DATE;
    CURS_COMENCAT EXCEPTION;
    CALENDARI_NO_TROBAT EXCEPTION;

    BEGIN
        
        BEGIN
        
            -- Comprova que el curs no hagi comenat o acabat
            SELECT dataInici, dataFi INTO v_dataInici, v_dataFi FROM CURS WHERE idCurs=v_curs;
            IF TO_DATE (v_dataInici, 'DD/MM/YYYY') < TO_DATE (SYSDATE, 'DD/MM/YYYY') THEN RAISE CURS_COMENCAT;
            END IF;
            
            -- elimina les entrades que coincideixen
            DELETE FROM CALENDARI WHERE curs=v_curs AND assignatura=v_assignatura AND diaSetmana=v_dia AND hora=v_hora;
            IF sql%rowcount = 0 THEN RAISE CALENDARI_NO_TROBAT;
            END IF;
            
            RSP:='OK';
            COMMIT;
            
        EXCEPTION
        
        WHEN CALENDARI_NO_TROBAT THEN
            RSP:='ERROR: CALENDARI_NO_TROBAT :  No s''ha trobat cap entrada similar al calendari';
            ROLLBACK;
        
        WHEN CURS_COMENCAT THEN
            RSP:='ERROR: CURS_COMENCAT : El curs ja ha comenat o finalitzat';
            ROLLBACK;
            
        WHEN OTHERS THEN
            RSP:='ERROR: ' || SQLERRM;
            ROLLBACK;
        END;
        -- anotar a la taula de log
        textLog:=('Curs=' || v_curs || 'Assignatura='|| v_assignatura || ', diaSetmana=' || v_dia); 
        insert_log('Baixa_Calendari', textLog, '', rsp);
    
    END;
/	
	
-- Baixa Professor
CREATE OR REPLACE PROCEDURE Baixa_Professor (v_nExpProfessor IN PROFESSOR.nExpProfessor%TYPE,
             rsp OUT NOCOPY VARCHAR2) IS
             
    textLog VARCHAR2(255 CHAR);
    v_tutorCurs PROFESSOR.tutorCurs%TYPE;
    PROFESSOR_NO_TROBAT EXCEPTION;
    PROFESSOR_ES_TUTOR EXCEPTION;
    rsp2 VARCHAR2(255 CHAR);
    BEGIN
        BEGIN
            -- Comprova que no es tutor de cap curs
            SELECT tutorCurs INTO v_tutorCurs FROM PROFESSOR WHERE nExpProfessor=v_nExpProfessor; 
            IF v_tutorCurs IS NOT NULL THEN RAISE PROFESSOR_ES_TUTOR;
            END IF;
        
            DELETE FROM PROFESSOR WHERE nExpProfessor=v_nExpProfessor;   
            IF sql%rowcount = 0 THEN RAISE PROFESSOR_NO_TROBAT;
            END IF;
            
            --Actualitzaci mdul estadstic 
            Gestio_Estadistic ('Professor',v_nExpProfessor ,NULL,NULL,'Baixa_Professor',NULL, RSP2);
            
            rsp:='OK';  
            COMMIT;

        EXCEPTION
            WHEN PROFESSOR_NO_TROBAT THEN
                rsp:= 'ERROR: PROFESSOR_NO_TROBAT : El professor no existeix';
                ROLLBACK; 
            WHEN PROFESSOR_ES_TUTOR THEN
                rsp:= 'ERROR: PROFESSOR_ES_TUTOR : El professor s tutor d''un curs';
                ROLLBACK; 
            WHEN OTHERS THEN 
                rsp:='ERROR:'||SQLERRM;
                ROLLBACK;
        END;    
            -- anotar a la taula de log
        textLog:=('nExpProfessor' || v_nExpProfessor); 
        insert_log('Baixa_Professor', textLog, '', rsp);
    END;    
 /   
-- Baixa Sanci
CREATE OR REPLACE
PROCEDURE Baixa_Sancio (v_idSancio IN SANCIO.idSancio%TYPE,
            rsp OUT NOCOPY VARCHAR2) IS
            
    DATAHORA_INCORRECTA EXCEPTION;     
    v_data date;
    textLog VARCHAR2(255 CHAR);
    cursRow curs%ROWTYPE;
    v_Curs SANCIO.curs%TYPE;
    
    BEGIN
        BEGIN
            SELECT curs INTO v_Curs FROM SANCIO WHERE idSancio=v_idSancio;
        
            -- dataHora est dins de les dates del curs
            SELECT * INTO cursRow FROM CURS WHERE idCurs = v_curs;
            IF cursRow.dataFi < SYSDATE THEN RAISE DATAHORA_INCORRECTA;
            END IF;
            
            -- Deixem com a no sancionades les amonestacions amb sancio=v_idSancio            
            UPDATE AMONESTACIO SET sancio=NULL WHERE sancio=v_idSancio;
        
			--Actualitzaci mdul estadstic 
            Gestio_Estadistic (NULL,NULL,NULL,NULL,'Baixa_Sancio',v_idSancio, RSP);
		
            DELETE FROM SANCIO WHERE idSancio=v_idSancio;

            rsp:='OK';  
            COMMIT;

        EXCEPTION
            WHEN DATAHORA_INCORRECTA THEN
                rsp:= 'ERROR: DATAHORA_INCORRECTA : La data i/o l''hora sn incorrectes';
                ROLLBACK; 
            WHEN NO_DATA_FOUND THEN
                rsp:= 'ERROR: NO_DATA_FOUND : Dades no trobades';
                ROLLBACK; 
            WHEN OTHERS THEN 
                rsp:='Error:'||SQLERRM;
                ROLLBACK;
        END;    
               -- anotar a la taula de log
        textLog:=('idSancio=' || v_idSancio); 
        insert_log('Baixa_Sancio', textLog, '', rsp);
    END;
/
-- Baixa Amonestaci
CREATE OR REPLACE PROCEDURE Baixa_Amonestacio (v_idAmon IN AMONESTACIO.idAmon%TYPE,
            rsp OUT NOCOPY VARCHAR2) IS
            
    DATAHORA_INCORRECTA EXCEPTION;     
    textLog VARCHAR2(255 CHAR);
    cursRow curs%ROWTYPE;
    amoRow AMONESTACIO%ROWTYPE;
    calSancioRSP VARCHAR2(255 CHAR);
    
    BEGIN
      
        BEGIN
             SELECT * INTO amoRow FROM AMONESTACIO WHERE idAmon=v_idAmon;  
            -- El curs ha finalitzat
            SELECT * INTO cursRow FROM CURS WHERE idCurs = amoRow.curs;
            IF (cursRow.dataFi < amoRow.data) THEN RAISE DATAHORA_INCORRECTA;
            END IF;
            
            -- Comprovem si hi ha una amonestaci del mateix tipus per sustituir aquesta i si no es dona de baixa la sanci
            IF amoRow.sancio IS NOT NULL THEN 
                 UPDATE AMONESTACIO SET sancio=amoRow.sancio WHERE nExpAlumne=amoRow.nExpAlumne AND curs=amoRow.CURS AND sancio=NULL 
                         AND tAmon=amoRow.tAmon AND rownum=1;
                 IF sql%rowcount = 0 THEN
                    Baixa_Sancio(amoRow.sancio, rsp);
                    rsp:='OK';
                 END IF;
            END IF; 

            --Actualitzaci mdul estadstic 
            Gestio_Estadistic (NULL,NULL,NULL,NULL,'Baixa_Amonestacio',v_idAmon, RSP);
			
            -- Esborrem l'amonestaci
            DELETE FROM AMONESTACIO WHERE idAmon=v_idAmon;

            rsp:='OK';  
            COMMIT;

        EXCEPTION

            WHEN DATAHORA_INCORRECTA THEN
                rsp:= 'ERROR: DATAHORA_INCORRECTA : La data i/o l''hora sn incorrectes';
                ROLLBACK; 
            WHEN NO_DATA_FOUND THEN
                rsp:= 'ERROR: NO_DATA_FOUND : Dades no trobades';
                ROLLBACK; 
            WHEN OTHERS THEN 
                rsp:='ERROR:'||SQLERRM;
                ROLLBACK;
        END;    
               -- anotar a la taula de log
        textLog:=('idAmon=' || v_idAmon); 
        insert_log('Baixa_Amonestacio', textLog, '', rsp);
    END;
/            
-- Baixa Tipus d'amonestaci
CREATE OR REPLACE PROCEDURE Baixa_TAmonestacio (v_IdTipusAmo IN T_AMONESTACIO.IdTipusAmo%TYPE,
             rsp OUT NOCOPY VARCHAR2) IS
             
    textLog VARCHAR2(255 CHAR);
    T_AMONESTACIO_NO_TROBAT EXCEPTION;
    
    BEGIN        
        BEGIN
        
            DELETE FROM T_AMONESTACIO WHERE v_IdTipusAmo=IdTipusAmo;
            -- Control del nombre de registres esborrats
            IF sql%rowcount = 0 THEN RAISE T_AMONESTACIO_NO_TROBAT; 
            END IF;

            rsp:='OK';  
            COMMIT;
            
        EXCEPTION
        
            WHEN T_AMONESTACIO_NO_TROBAT THEN
                rsp:= ('ERROR: T_AMONESTACIO_NO_TROBAT : El tipus d''amonestaci no existeix');
                ROLLBACK;
            WHEN OTHERS THEN 
                rsp:='Error:'||SQLERRM;
                ROLLBACK;    
        END;    
                -- anotar a la taula de log
        textLog:=('IdTipusAmo=' || v_IdTipusAmo); 
        insert_log('Baixa_TAmonestacio', textLog, '', rsp);
    END;
/    
    
-- Baixa Tipus Sanci
CREATE OR REPLACE PROCEDURE Baixa_TSancio (v_idTipusSancio IN T_SANCIO.idTipusSancio%TYPE,
            rsp OUT NOCOPY VARCHAR2) IS
    
  
    T_SANCIO_NO_TROBAT EXCEPTION;
    textLog VARCHAR2(255 CHAR);
    
    BEGIN        
        BEGIN
              
            DELETE FROM T_SANCIO WHERE idTipusSancio= v_idTipusSancio;
            IF sql%rowcount = 0 THEN RAISE T_SANCIO_NO_TROBAT;
            END IF; 

            rsp:='OK';  
            COMMIT;
            
        EXCEPTION
            WHEN T_SANCIO_NO_TROBAT THEN
                rsp:= 'ERROR: T_SANCIO_NO_TROBAT : El tipus de sanci no existeix';
                ROLLBACK;
                
            WHEN OTHERS THEN 
                rsp:='ERROR:'||SQLERRM;
                ROLLBACK;
        END;    
                -- anotar a la taula de log
        textLog:=('idTipusSancio=' || v_idTipusSancio); 
        insert_log('Baixa_TSancio', textLog, '', rsp);
    END;
/            
-- Modificaci d'alumne
CREATE OR REPLACE PROCEDURE Modificar_Alumne (v_nExpAlumne IN ALUMNE.nExpAlumne%TYPE,
            v_nif IN ALUMNE.nif%TYPE,
            v_nomCognoms IN ALUMNE.nomCognoms%TYPE, 
            v_adreca IN ALUMNE.adreca%TYPE,
            v_telefon IN ALUMNE.telefon%TYPE,
            v_email IN ALUMNE.email%TYPE, 
            rsp OUT NOCOPY VARCHAR2) IS

    textLog VARCHAR2(255 CHAR);
    ALUMNE_NO_TROBAT EXCEPTION;
    
    BEGIN
        BEGIN
        
            UPDATE ALUMNE SET NIF=v_nif,NOMCOGNOMS=v_nomCognoms, ADRECA=v_adreca, TELEFON=v_telefon, EMAIL=v_email WHERE nExpAlumne = v_nExpAlumne;
            IF sql%rowcount = 0 THEN RAISE ALUMNE_NO_TROBAT;
            END IF;

            rsp:='OK';  
            COMMIT;

        EXCEPTION
            WHEN ALUMNE_NO_TROBAT THEN
                rsp:= ('Error: ALUMNE_NO_TROBAT : L''alumne NO existeix');
                ROLLBACK; 
            WHEN OTHERS THEN 
                rsp:='Error:'||SQLERRM;
                ROLLBACK;
        END;    
              -- anotar a la taula de log
        textLog:=('nExpAlumne' || V_nExpAlumne || 'nif='|| v_nif || ', nomCognoms=' || v_nomCognoms || ', adreca=' ||v_adreca ||
             ', telefon=' || v_telefon || ', email=' ||  v_email); 
        insert_log('Alta_Alumne', textLog, '', rsp);
    END;
/
-- Modificar Assignatura
CREATE OR REPLACE PROCEDURE Modificar_Assignatura (v_IDASSIG IN ASSIGNATURA.IDASSIG%TYPE,
            v_NOMASSIGNATURA IN ASSIGNATURA.NOMASSIGNATURA%TYPE,
            v_DESCRIPCIO IN ASSIGNATURA.DESCRIPCIO%TYPE, 
             rsp OUT NOCOPY VARCHAR2) IS
             
    textLog VARCHAR2(255 CHAR);
    ASSIGNATURA_NO_TROBADA EXCEPTION;
    BEGIN        
        BEGIN
        
            UPDATE ASSIGNATURA SET NOMASSIGNATURA=v_NOMASSIGNATURA,DESCRIPCIO=v_DESCRIPCIO WHERE IDASSIG=v_IDASSIG;   
            IF sql%rowcount = 0 THEN RAISE ASSIGNATURA_NO_TROBADA;
            END IF;

            rsp:='OK';  
            COMMIT;
        EXCEPTION
            WHEN ASSIGNATURA_NO_TROBADA THEN
                rsp:= ('ERROR: ASSIGNATURA_NO_TROBADA : L''assignatura NO existeix');
                ROLLBACK; 
            WHEN OTHERS THEN 
                rsp:='ERROR:'||SQLERRM;
                ROLLBACK;    
        END;    
                -- anotar a la taula de log
        textLog:=('idAssig' || v_IDASSIG || 'nomAssignatura='|| v_NOMASSIGNATURA || ', descripcio=' || v_DESCRIPCIO); 
        insert_log('Modificar_Assignatura', textLog, '', rsp);
    END;
/    
-- Modificar Curs
CREATE OR REPLACE PROCEDURE Modificar_Curs (v_idCurs IN CURS.idCurs%TYPE,
            v_nomCurs IN CURS.nomCurs%TYPE,
            v_anyCurs IN CURS.anyCurs%TYPE, 
            v_descripcio IN CURS.descripcio%TYPE,
            v_dataInici IN CURS.dataInici%TYPE,
            v_dataFi IN CURS.dataFi%TYPE, 
            rsp OUT NOCOPY VARCHAR2) IS
             
    textLog VARCHAR2(255 CHAR);
    DATA_INICI_INCORRECTA EXCEPTION;
    DATA_FI_INCORRECTA EXCEPTION;
    ANYCURS_INCORRECTE EXCEPTION;
    CURS_NO_TROBAT EXCEPTION;
    
    BEGIN
        BEGIN
        
            IF TO_DATE (v_dataInici, 'DD/MM/YYYY') < TO_DATE(SYSDATE,'DD/MM/YYYY') THEN RAISE DATA_INICI_INCORRECTA;
            END IF;
            IF TO_DATE (v_dataFi, 'DD/MM/YYYY') < TO_DATE (v_dataInici, 'DD/MM/YYYY') THEN RAISE DATA_FI_INCORRECTA;
            END IF;
            IF TRUNC(TO_DATE (v_anyCurs, 'YYYY'), 'YEAR') < TRUNC(SYSDATE,'YEAR') THEN RAISE ANYCURS_INCORRECTE;
            END IF;
  
            UPDATE CURS SET NOMCURS=v_nomCurs, ANYCURS=v_anyCurs, DESCRIPCIO=v_descripcio, DATAINICI=v_dataInici, DATAFI=v_dataFi WHERE idCurs=v_idCurs;   
            IF sql%rowcount = 0 THEN RAISE CURS_NO_TROBAT;
            END IF;
            
            rsp:='OK';  
            COMMIT;

        EXCEPTION
            WHEN DATA_INICI_INCORRECTA THEN
                rsp:= ('ERROR: DATA_INICI_INCORRECTA : La data d''inici del curs no s correcta');
                 ROLLBACK; 
            WHEN DATA_FI_INCORRECTA THEN
                rsp:= ('ERROR: DATA_FI_INCORRECTA : La data de fi del curs no s correcta');
                 ROLLBACK; 
            WHEN ANYCURS_INCORRECTE THEN
                rsp:= ('ERROR: ANYCURS_INCORRECTE : L''any del curs no s correcte');
                 ROLLBACK;
            WHEN CURS_NO_TROBAT THEN
                rsp:= ('ERROR: CURS_NO_TROBAT : El curs no existeix');
                 ROLLBACK; 
            WHEN OTHERS THEN 
                rsp:='ERROR:'||SQLERRM;
                ROLLBACK;
        END;    
                    -- anotar a la taula de log
        textLog:=('idCurs' || v_idCurs || 'nomCurs='|| v_nomCurs || ', anyCurs=' || v_anyCurs || ', descripcio=' ||v_descripcio ||
             ', dataInici=' || v_dataInici || ', dataFi=' ||  v_dataFi); 
        insert_log('Modificar_Curs', textLog, '', rsp);
    END;
 /   
    
-- Modificar professor que imparteix una assignatura
CREATE OR REPLACE PROCEDURE Modificar_Imparteix (v_assignatura IN IMPARTEIX.assignatura%TYPE,
                                     v_professor IN IMPARTEIX.professor%TYPE,
                                     v_curs IN IMPARTEIX.curs%TYPE,
                                     rsp OUT NOCOPY VARCHAR2) IS
    textLog VARCHAR2(255 CHAR);
    CURS_FINALITZAT EXCEPTION;
    CURS_ASSIGNATURA_NO_TROBAT EXCEPTION;
    cursRow curs%ROWTYPE;
    
    BEGIN
    
        BEGIN
                -- El curs ha finalitzat?
            SELECT * INTO cursRow FROM CURS WHERE idCurs = v_Curs;
            IF cursRow.dataFi < SYSDATE THEN RAISE CURS_FINALITZAT;
            END IF;
            
			--Actualitzaci mdul estadstic 
            Gestio_Estadistic ('Professor',v_professor,'Curs',v_curs,'Modificar_Imparteix_Baixa',NULL, RSP); 
			
            UPDATE IMPARTEIX SET professor= v_professor WHERE curs=v_curs AND assignatura= v_assignatura;
            IF sql%rowcount = 0 THEN RAISE CURS_ASSIGNATURA_NO_TROBAT;
            END IF;
            
			--Actualitzaci mdul estadstic 
            Gestio_Estadistic ('Professor',v_professor,'Curs',v_curs,'Modificar_Imparteix_Alta',NULL, RSP); 
			
            rsp:='OK';  
            COMMIT;
        
        EXCEPTION
            WHEN CURS_FINALITZAT THEN
                rsp:= ('ERROR: CURS_FINALITZAT : El curs ja ha finalitzat');
                ROLLBACK; 
            WHEN CURS_ASSIGNATURA_NO_TROBAT THEN
                rsp:= ('ERROR: CURS_ASSIGNATURA_NO_TROBAT : L''assignatura no s''est impartint al curs');
                ROLLBACK; 
            WHEN OTHERS THEN 
                IF SQLCODE=-2291 THEN
                    rsp:='Alguna de les dades introduides no existeix';
                    ROLLBACK;
                ELSE
                    rsp:='ERROR:'||SQLERRM;
                    ROLLBACK;
                END IF;
        END;
            -- anotar a la taula de log
        textLog:=('assignatura='|| v_assignatura || ', professor=' || v_professor || ', curs=' || v_curs); 
        insert_log('Modificar_Imparteix', textLog, '', rsp);
        
    END;
 /  
    
-- Modificar Professor
CREATE OR REPLACE PROCEDURE Modificar_Professor (v_nExpProfessor IN PROFESSOR.nExpProfessor%TYPE,
            v_nif IN PROFESSOR.nif%TYPE,
            v_nomCognoms IN PROFESSOR.nomCognoms%TYPE, 
            v_adreca IN PROFESSOR.adreca%TYPE,
             v_telefon IN PROFESSOR.telefon%TYPE,
             v_email IN PROFESSOR.email%TYPE, 
             v_tutorCurs IN PROFESSOR.tutorCurs%TYPE,
             rsp OUT NOCOPY VARCHAR2) IS
             
    textLog VARCHAR2(255 CHAR);
    PROFESSOR_NO_TROBAT EXCEPTION;
    
    BEGIN
                
        BEGIN
        
            UPDATE PROFESSOR SET NIF=v_nif,NOMCOGNOMS=v_nomCognoms, ADRECA=v_adreca, TELEFON=v_telefon, EMAIL=v_email, TUTORCURS=v_tutorCurs WHERE nExpProfessor=v_nExpProfessor;   
            IF sql%rowcount = 0 THEN RAISE PROFESSOR_NO_TROBAT;
            END IF;
            rsp:='OK';  
            COMMIT;

        EXCEPTION
            WHEN PROFESSOR_NO_TROBAT THEN
                rsp:= 'ERROR: PROFESSOR_NO_TROBAT : El professor no existeix';
                ROLLBACK; 
            WHEN OTHERS THEN 
                IF SQLCODE=-2291 THEN
                    rsp:='ERROR: CURS_NO_EXISTEIX : El curs que es vol assignar no existeix';
                    ROLLBACK;
                ELSE
                    rsp:='ERROR:'||SQLERRM;
                    ROLLBACK;
                END IF;
        END;    
                -- anotar a la taula de log
        textLog:=('nExpProfessor' || v_nExpProfessor || 'nif='|| v_nif || ', nomCognoms=' || v_nomCognoms || ', adreca=' || v_adreca ||
             ', telefon=' || v_telefon || ', email=' ||  v_email || ', tutorCurs=' || v_tutorCurs); 
        insert_log('Modificar_Professor', textLog, '', rsp);
    END;
/   
-- Modificaci d'amonestacions
CREATE OR REPLACE PROCEDURE Modificar_Amonestacio (v_idAmon IN AMONESTACIO.idAmon%TYPE,
            v_tAmon IN AMONESTACIO.tAmon%TYPE,
            v_assignatura IN AMONESTACIO.assignatura%TYPE,
            v_profResponsable IN  AMONESTACIO.profResponsable%TYPE,
            v_curs IN AMONESTACIO.curs%TYPE,
            v_data IN AMONESTACIO.data%TYPE,
            v_hora IN AMONESTACIO.hora%TYPE,
            v_gravetat IN AMONESTACIO.gravetat%TYPE,
            v_comPares IN AMONESTACIO.comPares%TYPE,
            v_descripcio IN AMONESTACIO.descripcio%TYPE,
            rsp OUT NOCOPY VARCHAR2) IS
            
    DATAHORA_INCORRECTA EXCEPTION;
    AMONESTACIO_NO_TROBADA EXCEPTION;      
    dataHora date;
    textLog VARCHAR2(255 CHAR);
    cursRow curs%ROWTYPE;
    amoRow AMONESTACIO%ROWTYPE;
    calSancioRSP VARCHAR2(255 CHAR);
    RSP2 VARCHAR2(255 CHAR);
    
    BEGIN
        BEGIN
			SELECT * INTO amoRow FROM AMONESTACIO WHERE idAmon=v_idAmon;
            dataHora:= TO_DATE(v_data || ':' || v_hora, 'dd/mm/yyyy:hh:mi');
            
            -- dataHora no s una data futura
            IF dataHora > SYSDATE THEN RAISE DATAHORA_INCORRECTA;
            END IF;
            
            -- dataHora est dins de les dates del curs
            SELECT * INTO cursRow FROM CURS WHERE idCurs = v_curs;
            IF (TO_DATE(cursRow.dataInici,'dd/mm/yyyy') > dataHora) OR (TO_DATE(cursRow.dataFi,'dd/mm/yyyy') < dataHora) THEN RAISE DATAHORA_INCORRECTA;
            END IF;

            -- Si la dada que es modifica s el tipus d'amonestaci i ja tenia sanci, comprovem si hi ha 
            -- una amonestaci del tipus vell per sustituir aquesta i si no es dona de baixa la sanci
            IF (amoRow.tAmon <> v_tAmon) AND  amoRow.sancio IS NOT NULL THEN 
                 UPDATE (SELECT * FROM AMONESTACIO WHERE nExpAlumne=amoRow.nExpAlumne AND curs=v_curs AND sancio=NULL 
                         AND tAmon=amoRow.tAmon AND rownum=1 ORDER BY data, hora) SET sancio=amoRow.sancio;
                 IF sql%rowcount = 0 THEN
                    Baixa_Sancio(amoRow.sancio, rsp);
                    rsp:='OK';
                 END IF;
            END IF; 
      
            --Actualitzaci mdul estadstic 
            Gestio_Estadistic (NULL,NULL,NULL,NULL,'Modificar_Amonestacio_Baixa',v_idAmon, RSP2);
        
            -- Actualitzem l'amonestaci
            UPDATE AMONESTACIO SET tAmon=v_tAmon, assignatura=v_assignatura, profResponsable=v_profResponsable,
             curs=v_curs, data=v_data, hora=v_hora, gravetat=v_gravetat, comPares=v_comPares, descripcio=v_descripcio WHERE idAmon=v_idAmon;
             
            --Actualitzaci mdul estadstic 
            Gestio_Estadistic (NULL,NULL,NULL,NULL,'Modificar_Amonestacio_Alta',v_idAmon, RSP2);

            rsp:='OK';  
            COMMIT;
        
             -- Un cop tot ha anat b, es comprova si cal crear una sanci             
             CAL_SANCIO(amoRow.nExpAlumne,v_tAmon,v_data,v_hora,v_curs,calSancioRSP);

        EXCEPTION
            WHEN DATAHORA_INCORRECTA THEN
                rsp:= 'ERROR: DATAHORA_INCORRECTA : La data i/o l''hora sn incorrectes';
                ROLLBACK; 
            WHEN NO_DATA_FOUND THEN
                rsp:= 'ERROR: NO_DATA_FOUND : El curs referit no existeix';
                ROLLBACK; 
            WHEN AMONESTACIO_NO_TROBADA THEN
                rsp:= 'ERROR: AMONESTACIO_NO_TROBADA : L''amonestaci no existeix';
                ROLLBACK; 
            WHEN OTHERS THEN 
                IF SQLCODE=-2291 THEN
                    rsp:='Alguna de les dades introduides no existeix';
                    ROLLBACK;
                ELSE
                    rsp:='ERROR:'||SQLERRM;
                    ROLLBACK;
                END IF;
            --Actualitzaci mdul estadstic en cas de que falli l'update
            Gestio_Estadistic (NULL,NULL,NULL,NULL,'Modificar_Amonestacio_Alta',v_idAmon, RSP2);
        END;    
            -- anotar a la taula de log
        textLog:=('idAmon=' || v_idAmon|| ', tAmon=' || v_tAmon || ', assignatura=' || v_assignatura || ', profResponsable=' || v_profResponsable || ', curs=' ||  v_curs ||', data=' || v_data ||
             ', hora=' || v_hora || ', gravetat=' || v_gravetat || ', comPares=' || v_comPares || ', descripcio=' || v_descripcio); 
        insert_log('Modificar_Amonestacio', textLog, '', rsp);
    END;
/    
-- Modificaci de Sancions
CREATE OR REPLACE PROCEDURE Modificar_Sancio (v_idSancio IN SANCIO.idSancio%TYPE,
            v_data IN SANCIO.data%TYPE,
            v_hora IN SANCIO.hora%TYPE,
            v_curs IN SANCIO.curs%TYPE,
            v_descripcio IN SANCIO.descripcio%TYPE,
            rsp OUT NOCOPY VARCHAR2) IS
            
    DATAHORA_INCORRECTA EXCEPTION;  
    SANCIO_NO_TROBADA EXCEPTION; 
    SANCIO_AUTOMATICA EXCEPTION;    
    dataHora date;
    textLog VARCHAR2(255 CHAR);
    cursRow curs%ROWTYPE;
    v_tAmonestacio NUMBER;
    RSP2 VARCHAR2(255 CHAR);
    
    BEGIN
        BEGIN
            
            -- Comprovem que no es una sanci automtica
            SELECT tAmonestacio INTO v_tAmonestacio FROM T_SANCIO T INNER JOIN SANCIO S ON T.idTipusSancio=S.tipus 
                                WHERE S.IDSANCIO=v_idSancio;
            IF  v_tAmonestacio  IS NOT NULL THEN 
                RAISE SANCIO_AUTOMATICA;
             END IF;
            
            dataHora:= TO_DATE(v_data || ':' || v_hora, 'dd/mm/yyyy:hh:mi');
            
            -- dataHora no s una data futura
            IF dataHora > SYSDATE THEN RAISE DATAHORA_INCORRECTA;
            END IF;
            
            -- dataHora est dins de les dates del curs
            SELECT * INTO cursRow FROM CURS WHERE idCurs = v_curs;
            IF (TO_DATE(cursRow.dataInici,'dd/mm/yyyy') > dataHora) OR (TO_DATE(cursRow.dataFi,'dd/mm/yyyy') < dataHora) THEN RAISE DATAHORA_INCORRECTA;
            END IF;
            
            --Actualitzaci mdul estadstic 
            Gestio_Estadistic (NULL,NULL,NULL,NULL,'Modificar_Sancio_Baixa',v_idSancio, RSP2);
        
            UPDATE SANCIO SET data=v_data, hora=v_hora, curs=v_curs, descripcio=v_descripcio WHERE idSancio=v_idSancio;
            IF sql%rowcount = 0 THEN RAISE SANCIO_NO_TROBADA;
            END IF;
            
            --Actualitzaci mdul estadstic 
            Gestio_Estadistic (NULL,NULL,NULL,NULL,'Modificar_Sancio_Alta',v_idSancio, RSP2);
            
            rsp:='OK';  
            COMMIT;

        EXCEPTION
            WHEN SANCIO_AUTOMATICA THEN
                rsp:= 'SANCIO_AUTOMATICA : La sanci s''ha generat per una regla';
                ROLLBACK; 
            WHEN DATAHORA_INCORRECTA THEN
                rsp:= 'ERROR: DATAHORA_INCORRECTA : La data i/o l''hora sn incorrectes';
                ROLLBACK; 
            WHEN NO_DATA_FOUND THEN
                rsp:= 'ERROR: NO_DATA_FOUND : El curs referit no existeix';
                ROLLBACK; 
            WHEN SANCIO_NO_TROBADA THEN
                rsp:= 'ERROR: SANCIO_NO_TROBADA : La sanci no existeix';
                ROLLBACK; 
            WHEN OTHERS THEN 
                IF SQLCODE=-2291 THEN
                    rsp:='Alguna de les dades introduides no existeix';
                    ROLLBACK;
                ELSE
                    rsp:='Error:'||SQLERRM;
                    ROLLBACK;
                END IF;
                
                --Actualitzaci mdul estadstic 
                Gestio_Estadistic (NULL,NULL,NULL,NULL,'Modificar_Sancio_Alta',v_idSancio, RSP2);
        END;    
               -- anotar a la taula de log
        textLog:=('idSancio=' || v_idSancio || ', data=' || v_data ||
             ', hora=' || v_hora || ', curs=' ||  v_curs || ', descripcio=' || v_descripcio); 
        insert_log('Modificar_Sancio', textLog, '', rsp);
    END;
/    
    
-- Modificar Tipus d'amonestacions
CREATE OR REPLACE PROCEDURE Modificar_TAmonestacio (v_IdTipusAmo IN T_AMONESTACIO.IdTipusAmo%TYPE,
            v_tAmonestacio IN T_AMONESTACIO.tAmonestacio%TYPE,
            v_descripcio IN T_AMONESTACIO.descripcio%TYPE, 
             rsp OUT NOCOPY VARCHAR2) IS
             
    textLog VARCHAR2(255 CHAR);
    T_AMONESTACIO_NO_TROBAT EXCEPTION;
    
    BEGIN        
        BEGIN
        
            UPDATE T_AMONESTACIO SET TAMONESTACIO=v_tAmonestacio,DESCRIPCIO= v_descripcio WHERE v_IdTipusAmo=IdTipusAmo;
            -- Control del nombre de registres actualitzats
            IF sql%rowcount = 0 THEN RAISE T_AMONESTACIO_NO_TROBAT; 
            END IF;

            rsp:='OK';  
            COMMIT;
            
        EXCEPTION
            WHEN T_AMONESTACIO_NO_TROBAT THEN
                rsp:= ('ERROR: T_AMONESTACIO_NO_TROBAT : El tipus d''amonestaci no existeix');
                ROLLBACK;
            WHEN OTHERS THEN 
                rsp:='ERROR:'||SQLERRM;
                ROLLBACK;    
        END;    
                -- anotar a la taula de log
        textLog:=('IdTipusAmo=' || v_IdTipusAmo || 'tAmonestacio='|| v_tAmonestacio || ', descripcio=' || v_descripcio); 
        insert_log('Modificar_TAmonestacio', textLog, '', rsp);
    END;
/    
    
-- Modificar Tipus de Sancions
CREATE OR REPLACE PROCEDURE Modificar_TSancio (v_idTipusSancio IN T_SANCIO.idTipusSancio%TYPE,
            v_tSancio IN T_SANCIO.tSancio%TYPE,
            v_quanActiva IN T_SANCIO.quanActiva%TYPE, 
            v_tAmonestacio IN T_SANCIO.tAmonestacio%TYPE,
            v_operador IN T_SANCIO.operador%TYPE,
            v_nAmonestacions IN T_SANCIO.nAmonestacions%TYPE,
            rsp OUT NOCOPY VARCHAR2) IS

    OPERADOR_NO_VALID EXCEPTION;  
    T_SANCIO_NO_TROBAT EXCEPTION;
    textLog VARCHAR2(255 CHAR);
    
    BEGIN        
        BEGIN
        
            -- Comprovaci dels parmetres d'entrada
            IF v_operador NOT IN ('=','<','>') THEN RAISE OPERADOR_NO_VALID;
            END IF;
        
            UPDATE T_SANCIO SET TSANCIO=v_tSancio,QUANACTIVA=v_quanActiva,TAMONESTACIO=v_tAmonestacio,OPERADOR=v_operador,
                                NAMONESTACIONS=v_nAmonestacions WHERE idTipusSancio= v_idTipusSancio;
            IF sql%rowcount = 0 THEN RAISE T_SANCIO_NO_TROBAT;
            END IF; 

            rsp:='OK';  
            COMMIT;
            
        EXCEPTION
            WHEN T_SANCIO_NO_TROBAT THEN
                rsp:= 'ERROR: T_SANCIO_NO_TROBAT : El tipus de sanci no existeix';
                ROLLBACK;        
            WHEN OPERADOR_NO_VALID THEN
                rsp:= 'ERROR: OPERADOR_NO_VALID : L''operador ha de ser <, > o =';
                ROLLBACK;
            WHEN OTHERS THEN 
                IF SQLCODE=-2291 THEN
                    rsp:='ERROR: AMONESTACIO_NO_EXISTEIX : El tipus d''amonestaci que es vol assignar no existeix';
                    ROLLBACK;
                ELSE
                    rsp:='ERROR:'||SQLERRM;
                    ROLLBACK;
                END IF;    
        END;    
                -- anotar a la taula de log
        textLog:=('idTipusSancio=' || v_idTipusSancio || ', tSancio='|| v_tSancio || ', quanActiva=' || v_quanActiva || ', tAmonestacio='|| v_tAmonestacio ||
             ', operador=' || v_operador || ', nAmonestacions=' || v_nAmonestacions); 
        insert_log('Modificar_TSancio', textLog, '', rsp);
    END;
/    
-- Comprovar si cal crear una sanci
CREATE OR REPLACE PROCEDURE CAL_SANCIO(v_nExpAlumne IN SANCIO.nExpAlumne%TYPE,
            v_tAmonestacio IN AMONESTACIO.tAmon%TYPE,
            v_data IN SANCIO.data%TYPE,
            v_hora IN SANCIO.hora%TYPE,
            v_curs IN SANCIO.curs%TYPE,
            rsp OUT NOCOPY VARCHAR2) IS
              
    tSancioROW T_SANCIO%ROWTYPE;
    nMateixaAmonest NUMBER;
    calSancio BOOLEAN := FALSE;
    textLog VARCHAR2(255 CHAR);
    
    BEGIN            
        BEGIN
                        
            -- nAmonestacions de l'alumne, tipus i curs designats i que no hagin estat sancionades.
            SELECT COUNT(*) INTO nMateixaAmonest FROM AMONESTACIO WHERE nExpAlumne=v_nExpAlumne AND curs=v_curs AND sancio IS NULL AND tAmon=v_tAmonestacio; 
            -- Comprovem si hi ha un tipus de sanci pel tipus d'amonestaci i si compleix la regla.

            SELECT * INTO tSancioROW FROM T_SANCIO WHERE tAmonestacio = v_tAmonestacio AND ROWNUM=1 ORDER BY idTipusSancio;
            IF tSancioROW.operador ='<' AND nMateixaAmonest < tSancioROW.nAmonestacions THEN 
                    calSancio:= TRUE;
            ELSIF tSancioROW.operador ='>' AND nMateixaAmonest > tSancioROW.nAmonestacions THEN 
                    calSancio:= TRUE;
            ELSIF tSancioROW.operador ='=' AND nMateixaAmonest = tSancioROW.nAmonestacions THEN 
                    calSancio:= TRUE;   
            END IF;
             
            IF calSancio = TRUE THEN
                Alta_Sancio (v_nExpAlumne,tSancioROW.idTipusSancio, v_data,v_hora,v_curs,'', rsp);
            END IF;
            
            rsp:='OK';
            
            EXCEPTION
                WHEN OTHERS THEN 
                    rsp:='Error:'||SQLERRM;
                    ROLLBACK;
        END;
                -- anotar a la taula de log
        textLog:=('nExpAlumne='|| v_nExpAlumne || ', tipus=' || v_tAmonestacio || ', data=' || v_data ||
             ', hora=' || v_hora || ', curs=' ||  v_curs ); 
        insert_log('Cal_Sancio', textLog, '', rsp);
    END;
 /   
    
    
                
    

            